package com.memoir44.vassal;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.ScrollPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Random;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import javax.xml.transform.ErrorListener;

import VASSAL.Info;
import VASSAL.build.BadDataReport;
import VASSAL.build.GameModule;
import VASSAL.build.module.GameComponent;
import VASSAL.build.module.GameState;
import VASSAL.i18n.Resources;
import VASSAL.tools.BugUtils;
import VASSAL.tools.ErrorDialog;
import VASSAL.tools.imageop.Op;

public class M44ScenarioChooser extends JDialog {
	private JDialog thisFrame;
	
	private JPanel mainPanel;
	private JPanel listPanel;
	private JPanel filterPanel;
	private JPanel buttonPanel;
	
	private JTable scenarioList;
	private ScenarioTableModel scenarioModel;
	
	private JPanel jpText;
	private JTextField jtText;
	private String filterText = "";
	
	private JPanel jpFront;
	private JCheckBox[] jcFront;
	private boolean[] filterFronts = new boolean[5];
	private String[] frontNames = {"Western", "Eastern", "Pacific", "Mediterranean", "Other"};
	
	private JPanel jpExpansions;
	//private JCheckBox[] jcExpansions;
	private JComboBox jcExpansions;
	private boolean[] filterExpansions = new boolean[17];
	private int indexAirPackExpansion = -1;
	private String[] expansionNames = {"Base Game", "Terrain Pack", "Eastern", "Pacific", "Air Pack", "Mediterranean", "Hedgerow Hell", "Tigers in the Snow", "Sword of Stalingrad", "Disaster at Dieppe", "Breakthrough", "Winter Wars", "Campaign Book Vol. 1", "Campaign Book Vol. 2", "Vercors Campaign", "Audie Murphy", "Invasion of Crete"};
	private String[] expansionCodes = {"ST", "TP", "EF", "PT", "AA", "MT", "HH", "TS", "SS", "DD", "BT", "WW", "CB", "C2", "VC", "DM", "IC"};
	private String[] expansionMenu = {"All", "Base Game", "Terrain Pack", "Eastern", "Pacific", "Air Pack", "Mediterranean", "Hedgerow Hell", "Tigers in the Snow", "Sword of Stalingrad", "Disaster at Dieppe", "Breakthrough", "Winter Wars", "Campaign Book Vol. 1", "Campaign Book Vol. 2", "Vercors Campaign", "Audie Murphy", "Invasion of Crete"};
	
	private JPanel jpType;
	private JCheckBox[] jcType;
	private boolean[] filterTypes = new boolean[16];
	private String[] typeNames = {"Inland", "Beach", "Winter", "Desert", "BT-Inland", "BT-Beach", "BT-Winter", "BT-Desert", "OL-Inland", "OL-Beach", "OL-Winter", "OL-Desert", "BL-Inland", "BL-Beach", "BL-Winter", "BL-Desert"};
	
	private JPanel jpYear;
	private JCheckBox[] jcYear;
	private boolean[] filterYears = new boolean[8];
	private String[] yearNames = {"1939", "1940", "1941", "1942", "1943", "1944", "1945", "Other"};
	private String[] monthNames = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
	
	private JCheckBox jcOfficialScenarios;
	private boolean filterOfficial = false;
	private JCheckBox jcSFTF;
	private boolean filterSFTF = false;
	
	private JPanel numScenPanel;
	private JLabel jlNumScen;
	
	private JButton randomButton;
	private JButton cancelButton;
	private JButton selectButton;
	
	private JPanel previewPanel;
	private JComponent previewPane;
	private Image previewImage = null;
	
	private GameState myGameState;
	private Vector<M44Setup> setups = new Vector<M44Setup>();
	
	private Vector<M44Series> series = new Vector<M44Series>();
	
	public M44ScenarioChooser(GameState gameState) {
		myGameState = gameState;
		
		jbInit();
	}
	
	private void jbInit() {
		thisFrame = this;
		
		Dimension dim = getToolkit().getScreenSize();
		
		int width, height;
		
		width = dim.width - 75;
		height = dim.height - 75;
		
		if (width > 1280)
			width = 1280;
		
		this.setTitle("Choose a Memoir '44 Scenario");
		this.setSize(width, height);
		this.setLocation((dim.width - width) / 2, (dim.height - height) / 2);
		this.setResizable(true);
		
		mainPanel = new JPanel(new BorderLayout(5, 5));
		mainPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		listPanel = new JPanel(new BorderLayout());
		
		filterPanel = new JPanel();
		filterPanel.setLayout(new BoxLayout(filterPanel, BoxLayout.Y_AXIS));
		
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		
		Collection<GameComponent> comps = myGameState.getGameComponents();
		
		Iterator<GameComponent> it = comps.iterator();
		
		do {
			GameComponent comp = it.next();
			if (comp instanceof M44Setup) {
				setups.add((M44Setup) comp);
			}
			if (comp instanceof M44Series) {
				series.add((M44Series) comp);
			}
		} while (it.hasNext());
		
		if (series.size() == 0)
		{
			int num = expansionNames.length;
			for (int i = 0; i < num; i++)
				series.add(new M44Series(expansionNames[i], expansionCodes[i]));
		}
		
		filterExpansions = new boolean[series.size()];
		
		Comparator<M44Setup> sortScenarios = new ScenarioSorter();
		
		Collections.sort(setups, sortScenarios);
		
		Comparator<M44Series> sortSeries = new SeriesSorter();
		
		Collections.sort(series, sortSeries);
		
		for (int i = series.size() - 1; i > 0; i--)
		{
			if (series.get(i-1).getConfigureName().equals(series.get(i).getConfigureName()))
				series.remove(i);
		}
		
		for (int i = 0; i < series.size(); i++)
			if (series.get(i).prefix.equals("AA"))
				indexAirPackExpansion = i;
		
		expansionMenu = new String[series.size()+1];
		expansionMenu[0] = "All";
		int num = series.size();
		for (int i = 0; i < num; i++)
			expansionMenu[i+1] = series.get(i).getAttributeValueString(M44Series.NAME);
		
		numScenPanel = new JPanel();
		numScenPanel.setBorder(BorderFactory.createTitledBorder("Scenarios"));
		numScenPanel.setMaximumSize(new Dimension(32000, 24));
		jlNumScen = new JLabel("Number-of-Scenarios");
		numScenPanel.add(jlNumScen);
		
		filterPanel.add(numScenPanel);
		
		jpText = new JPanel();
		jpText.setBorder(BorderFactory.createTitledBorder("Filter"));
		jpText.setLayout(new BoxLayout(jpText, BoxLayout.Y_AXIS));
		//jpText.setLayout(new GridLayout(3, 1));
		
		JPanel jpTextField = new JPanel(new GridLayout(3, 1));
		
		jtText = new JTextField("                                                        ");
		jtText.setMinimumSize(new Dimension(200, 25));
		jtText.setMaximumSize(new Dimension(32000, 25));
		jtText.getDocument().addDocumentListener(new DocumentListener() {
			public void removeUpdate(DocumentEvent e) {
				filterText = jtText.getText().toLowerCase();

				filterScenarioList();
			}
			
			public void insertUpdate(DocumentEvent e) {
				filterText = jtText.getText().toLowerCase();

				filterScenarioList();
			}
			
			public void changedUpdate(DocumentEvent e) {
				filterText = jtText.getText().toLowerCase();

				filterScenarioList();
			}
		});
		jpTextField.add(jtText);
		
		//jpText.add(jtText); //jpTextField);
		
		JPanel pOff = new JPanel();
		pOff.setLayout(new BoxLayout(pOff, BoxLayout.Y_AXIS));
		pOff.setMaximumSize(new Dimension(32000, 24));
		
		jcOfficialScenarios = new JCheckBox("Official Scenarios");
//		jcOfficialScenarios.setSelected(true);
		jcOfficialScenarios.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				filterOfficial = jcOfficialScenarios.isSelected();
				
				filterScenarioList();
			}
		});
		
		//pOff.add(jcOfficialScenarios);
		//jpText.add(jcOfficialScenarios); //pOff);
		jpTextField.add(jcOfficialScenarios);
		
		pOff = new JPanel();
		pOff.setLayout(new BoxLayout(pOff, BoxLayout.Y_AXIS));
		pOff.setMaximumSize(new Dimension(32000, 24));
		
		jcSFTF = new JCheckBox("SFTF (Unofficial)");
//		jcSFTF.setSelected(true);
		jcSFTF.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				filterSFTF = jcSFTF.isSelected();
				
				filterScenarioList();
			}
		});
		
		//pOff.add(jcSFTF);
		//jpText.add(jcSFTF); //pOff);
		jpTextField.add(jcSFTF);
		
		jpText.add(jpTextField);
		jpText.add(Box.createVerticalGlue());
		
		filterPanel.add(jpText);
		
		jpFront = new JPanel();
		jpFront.setBorder(BorderFactory.createTitledBorder("Front"));
		jpFront.setLayout(new BoxLayout(jpFront, BoxLayout.Y_AXIS));
		
		jcFront = new JCheckBox[frontNames.length];
		for (int i = 0; i < frontNames.length; i++) {
			final int j = i;
			JPanel p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
			p.setMaximumSize(new Dimension(32000, 24));
			jcFront[i] = new JCheckBox(frontNames[i]);
//			jcFront[i].setSelected(true);
//			filterFronts[i] = true;
			jcFront[i].addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					filterFronts[j] = jcFront[j].isSelected();
					
					filterScenarioList();
				}
			});
			p.add(jcFront[i]);
			jpFront.add(p);
		}
		
		filterPanel.add(jpFront);
		
		jpExpansions = new JPanel();
		jpExpansions.setBorder(BorderFactory.createTitledBorder("Series")); // Expansions
		jpExpansions.setLayout(new BoxLayout(jpExpansions, BoxLayout.Y_AXIS));
		
		jcExpansions = new JComboBox(expansionMenu);
		jcExpansions.setEditable(false);
		jcExpansions.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				int idx = jcExpansions.getSelectedIndex();
				
				for (int i = 0; i < filterExpansions.length; i++)
					filterExpansions[i] = ((idx-1) == i);
				
				filterScenarioList();
			}
		});
		jpExpansions.add(jcExpansions);
		
		filterPanel.add(jpExpansions);
		
		jpType = new JPanel();
		jpType.setBorder(BorderFactory.createTitledBorder("Type"));
		jpType.setLayout(new BoxLayout(jpType, BoxLayout.Y_AXIS));
		
		jcType = new JCheckBox[typeNames.length];
		for (int i = 0; i < typeNames.length; i++) {
			final int j = i;
			JPanel p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
			p.setMaximumSize(new Dimension(32000, 24));
			jcType[i] = new JCheckBox(typeNames[i]);
//			jcType[i].setSelected(true);
//			filterTypes[i] = true;
			jcType[i].addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					filterTypes[j] = jcType[j].isSelected();
					
					filterScenarioList();
				}
			});
			p.add(jcType[i]);
			jpType.add(p);
		}
		
		filterPanel.add(jpType);
		
		jpYear = new JPanel();
		jpYear.setBorder(BorderFactory.createTitledBorder("Year"));
		jpYear.setLayout(new BoxLayout(jpYear, BoxLayout.Y_AXIS));
		
		jcYear = new JCheckBox[yearNames.length];
		for (int i = 0; i < yearNames.length; i++) {
			final int j = i;
			JPanel p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
			p.setMaximumSize(new Dimension(32000, 24));
			jcYear[i] = new JCheckBox(yearNames[i]);
//			jcYear[i].setSelected(true);
//			filterYears[i] = true;
			jcYear[i].addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					filterYears[j] = jcYear[j].isSelected();
					
					filterScenarioList();
				}
			});
			p.add(jcYear[i]);
			jpYear.add(p);
		}
		
		filterPanel.add(jpYear);
				
		filterPanel.add(Box.createVerticalGlue());
		
		JScrollPane filterScroller = new JScrollPane(filterPanel);
		filterScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		filterScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		scenarioModel = new ScenarioTableModel();
		scenarioList = new JTable(scenarioModel);
		filterScenarioList();
		
		JTableHeader header = scenarioList.getTableHeader();
	    header.setUpdateTableInRealTime(true);
	    header.addMouseListener(scenarioModel.new ColumnListener(scenarioList));
	    header.setReorderingAllowed(true);
		
		scenarioList.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).
        	put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
            	"pressSelectScenario");
		
		scenarioList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				int row = scenarioList.getSelectedRow();
				String imageName = "";
				
				if (row >= 0)
					// load preview pane image
					if (scenarioModel.getRow(row).fileName != null)
						imageName = scenarioModel.getRow(row).getAttributeValueString(M44Setup.FILE) + ".jpg";
				
				if (imageName.length() > 0)
					previewImage = Op.load(imageName).getImage();
				else
					previewImage = null;
				
				previewPane.repaint();
			}
		});
		scenarioList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					int row = ((JTable) e.getSource()).getSelectedRow();
					M44Setup setup = scenarioModel.getRow(row);
					
					launchSetup(setup);
				}
			}
		});
		int[] widths = {85, 0, 200, 105, 60, 350, 105};
		
		for (int i = 0; i < widths.length; i++)
			if (widths[i] > 0) {
				scenarioList.getColumnModel().getColumn(i).setPreferredWidth(widths[i]);
				scenarioList.getColumnModel().getColumn(i).setMaxWidth(widths[i]);
			}
		scenarioList.getColumnModel().getColumn(1).setMinWidth(250);
		
		JScrollPane scScenarioList = new JScrollPane(scenarioList);
		listPanel.add(scScenarioList, BorderLayout.CENTER);
		
		randomButton = new JButton("Pick Random");
		randomButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				pressRandomScenario();
			}
		});
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				thisFrame.dispose();
			}
		});
		selectButton = new JButton("Load");
		selectButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				pressSelectScenario();
			}
		});
		selectButton.setDefaultCapable(true);
		
		JButton btCheckNone = new JButton("Select None");
		btCheckNone.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for (int i = 0; i < frontNames.length; i++)
					if (jcFront[i].isSelected()) {
						jcFront[i].setSelected(false);
						filterFronts[i] = false;
					}
				
				for (int i = 0; i < typeNames.length; i++)
					if (jcType[i].isSelected()) {
						jcType[i].setSelected(false);
						filterTypes[i] = false;
					}
				
				for (int i = 0; i < filterExpansions.length; i++)
					//if (jcExpansions[i].isSelected()) {
					//	jcExpansions[i].setSelected(false);
						filterExpansions[i] = false;
					//}
				jcExpansions.setSelectedIndex(0);
				
				for (int i = 0; i < yearNames.length; i++)
					if (jcYear[i].isSelected()) {
						jcYear[i].setSelected(false);
						filterYears[i] = false;
					}
				
				filterScenarioList();
			}
		});
		
		JButton btCheckAll = new JButton("Select All");
		btCheckAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for (int i = 0; i < frontNames.length; i++)
					if (!jcFront[i].isSelected()) {
						jcFront[i].setSelected(true);
						filterFronts[i] = true;
					}
				
				for (int i = 0; i < typeNames.length; i++)
					if (!jcType[i].isSelected()) {
						jcType[i].setSelected(true);
						filterTypes[i] = true;
					}
				
				for (int i = 0; i < filterExpansions.length; i++)
					//if (!jcExpansions[i].isSelected()) {
						//jcExpansions[i].setSelected(true);
						filterExpansions[i] = false; //true;
					//}
				jcExpansions.setSelectedIndex(0);
				
				for (int i = 0; i < yearNames.length; i++)
					if (!jcYear[i].isSelected()) {
						jcYear[i].setSelected(true);
						filterYears[i] = true;
					}
				
				filterScenarioList();
			}
		});
		
		buttonPanel.add(btCheckNone);
		buttonPanel.add(Box.createHorizontalStrut(10));
		buttonPanel.add(btCheckAll);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(randomButton);
		buttonPanel.add(Box.createHorizontalStrut(10));
		buttonPanel.add(cancelButton);
		buttonPanel.add(Box.createHorizontalStrut(10));
		buttonPanel.add(selectButton);
		
		previewPanel = new JPanel(new BorderLayout());
		previewPanel.setBorder(BorderFactory.createTitledBorder("Preview"));
		
		previewPane = new JComponent() {
			@Override
			public void paintComponent(Graphics g) {
				if (previewImage != null)
					g.drawImage(previewImage, 0, 0, null);
			}
		};
		previewPane.setPreferredSize(new Dimension(350, 250));
		previewPane.setMinimumSize(new Dimension(350, 250));
		previewPanel.add(previewPane, BorderLayout.CENTER);
		
		listPanel.add(previewPanel, BorderLayout.SOUTH);
	
		mainPanel.add(listPanel, BorderLayout.CENTER);
		mainPanel.add(filterScroller, BorderLayout.WEST);
		mainPanel.add(buttonPanel, BorderLayout.SOUTH);
		
		thisFrame.getContentPane().add(mainPanel, BorderLayout.CENTER);
		thisFrame.getRootPane().setDefaultButton(selectButton);
		
		jtText.setText("");
	}
	
	protected void pressSelectScenario() {
		int row = scenarioList.getSelectedRow();
		if (row >= 0)
			launchSetup(scenarioModel.getRow(row));
	}
	
	protected void filterScenarioList() {
		Vector<M44Setup> list = new Vector<M44Setup>();
		
		boolean doOriginFilter = (filterOfficial || filterSFTF);
		
		boolean doFrontFilter = false;
		
		for (int j = 0; j < frontNames.length; j++)
			if (filterFronts[j] == true)
				doFrontFilter = true;
		
		boolean doExpansionFilter = false;
		
		for (int j = 0; j < series.size(); j++)
			if (filterExpansions[j] == true)
				doExpansionFilter = true;

		boolean doTypeFilter = false;
		
		for (int j = 0; j < typeNames.length; j++)
			if (filterTypes[j] == true)
				doTypeFilter = true;

		boolean doYearFilter = false;
		
		for (int j = 0; j < yearNames.length; j++)
			if (filterYears[j] == true)
				doYearFilter = true;
		
//		boolean doFrontFilter = true;
//		boolean doExpansionFilter = true;
//		boolean doTypeFilter = true;
//		boolean doYearFilter = true;
		
		for (int i = 0; i < setups.size(); i++) {
			M44Setup item = setups.get(i);
			
			String frontName = item.front;
			String yearName = item.year;
			String typeName = item.scentype;
			String preCode = "";
			if (item.m44code.length() >= 2)
				preCode = item.m44code.substring(0, 2);
			
			String errorLogPath =
			      new File(Info.getConfDir(), "errorLog").getAbsolutePath();
			
			boolean isAirPack = false;
			if (preCode.equals("AW") || preCode.equals("AE") || preCode.equals("AM") || preCode.equals("AP"))
				isAirPack = true;
			boolean filterAirPack = false;
			
			boolean isOfficial = false;
			
			if (item.official != null)
				isOfficial = item.official;
			
			boolean found = false;
			
			for (int j = 0; j < frontNames.length; j++)
				if (frontNames[j].equals(frontName))
					found = true;
			if (!found)
				frontName = frontNames[frontNames.length-1];
			
			found = false;
			
			for (int j = 0; j < series.size(); j++) {
				M44Series serie = series.get(j);
				
//				FileWriter out;
//				try {
//					out = new FileWriter(errorLogPath, true);
//					out.write((j+1) + ". CODE = " + serie.prefix + " == " + preCode + "\n");
//					out.close();
//				} catch (IOException e) {
//					JOptionPane.showMessageDialog(null, e.getMessage());
//					e.printStackTrace();
//				}
				
				if (serie.prefix.equals(preCode))
					found = true;
			}
			if (!found)
				if (preCode.length() >= 1 && preCode.substring(0, 1).equals("A"))
					filterAirPack = true;
				else
					preCode = "";
			
			found = false;
			yearName = "19"+yearName;
			
			for (int j = 0; j < yearNames.length; j++)
				if (yearNames[j].equals(yearName))
					found = true;
			if (!found)
				yearName = yearNames[yearNames.length-1];
			
			boolean keepItem = true;
			
			if (doOriginFilter && isOfficial && !filterOfficial)
				keepItem = false;
			
			if (doOriginFilter && !isOfficial && !filterSFTF)
				keepItem = false;
			
			if (keepItem && doFrontFilter) {
				keepItem = false;
				for (int j = 0; j < frontNames.length; j++)
					if ((frontNames[j].equals(frontName)) && (filterFronts[j] == true))
						keepItem = true;
			}
			
			if (keepItem && doExpansionFilter) {
				keepItem = false;
				
				if (filterAirPack && filterExpansions[indexAirPackExpansion] == true) {
					if (isAirPack)
						keepItem = true;
				} else {
					for (int j = 0; j < series.size(); j++)
						if ((series.get(j).prefix.equals(preCode)) && (filterExpansions[j] == true))
							keepItem = true;
				}
			}
			
			if (keepItem)
				if (doTypeFilter) {
					keepItem = false;
					for (int j = 0; j < typeNames.length; j++)
						if ((typeNames[j].equals(typeName)) && (filterTypes[j] == true))
							keepItem = true;
				}
			
			if (keepItem)
				if (doYearFilter) {
					keepItem = false;
					for (int j = 0; j < yearNames.length; j++)
						if ((yearNames[j].equals(yearName)) && (filterYears[j] == true))
							keepItem = true;
				}
			
			if (keepItem)
				if (filterText.length() > 0) {
					keepItem = false;
					
					String name = item.getConfigureName();
					String set = "";
					if (item.set != null) set = item.set;
					String operation = ""; if (item.operation != null) operation = item.operation;
					
					if (name.toLowerCase().indexOf(filterText) >= 0) keepItem = true;
					if (set.toLowerCase().indexOf(filterText) >= 0) keepItem = true;
					if (operation.toLowerCase().indexOf(filterText) >= 0) keepItem = true;
				}
			
			if (keepItem)
				list.add(item);
		}
		
		scenarioModel.setData(list);
		scenarioModel.sortData(scenarioList);
		
		jlNumScen.setText("" + list.size() + " of " + setups.size());
	}
	
	protected void pressRandomScenario() {
		Random r = new Random();
		
		int max = scenarioModel.getRowCount();
		int choice = r.nextInt(max);
		scenarioList.getSelectionModel().setSelectionInterval(choice, choice);
		scenarioList.scrollRectToVisible(new Rectangle(scenarioList.getCellRect(choice, 0, true)));
	}

	protected void launchSetup(M44Setup setup) {
		if (setup.launch())
			thisFrame.dispose();
	}

	private class ScenarioTableModel extends AbstractTableModel {
		private static final long serialVersionUID = 1L;
		private Vector<M44Setup> data = new Vector<M44Setup>();
		private String[] columnNames = {"Type", "Title", "Set", "Front", "Date", "Operation", "Code"};
		
		protected int sortCol = 1;
		protected boolean isSortAsc = true;
		protected boolean sortByID = false;
		
		public void setData(Vector<M44Setup> newData) {
			data.clear();
			data = newData;
			
			fireTableDataChanged();
		}
		
		public String getColumnName(int col) {
			return columnNames[col];
		}
		
		public int getColumnCount() {
			return 7;
		}

		public int getRowCount() {
			return data.size();
		}
		
		public M44Setup getRow(int row) {
			return data.get(row);
		}

		public Object getValueAt(int row, int col) {
			switch (col) {
			case 0:
				return data.get(row).scentype;
			case 1:
				String name = data.get(row).getConfigureName();
//				if (data.get(row).airpack)
//					name += " [AIR PACK]";
				return name;
			case 2:
				if ((data.get(row).set == null) || (data.get(row).set.length() == 0))
					return "";
				else
					return "["+data.get(row).set+"]";
			case 3:
				return data.get(row).front;
			case 4:
				return data.get(row).getDate();
			case 5:
				return data.get(row).operation;
			case 6:
				String code = "";
				if (data.get(row).m44code != null)
					code = data.get(row).m44code;
				String dow = "";
				if (data.get(row).dowid != null)
					dow = data.get(row).dowid;
				
				if ((code.length() > 0) && (dow.length() > 0)) {
					code = code + " #" + dow;
				} else if (dow.length() > 0)
					code = "#" + dow;
				
				return code;
			default:
				return "";
			}
		}
		
		public void sortData(JTable table) {
			Collections.sort(data,new MyComparator(isSortAsc, sortCol));
	    	table.tableChanged(new TableModelEvent(ScenarioTableModel.this));
	    	table.repaint();
		}
		
		class ColumnListener extends MouseAdapter {
		    protected JTable table;

		    public ColumnListener(JTable t) {
		      table = t;
		    }

		    public void mouseClicked(MouseEvent e) {
		    	TableColumnModel colModel = scenarioList.getColumnModel();
		    	int columnModelIndex = colModel.getColumnIndexAtX(e.getX());
		    	int modelIndex = colModel.getColumn(columnModelIndex)
		    	.getModelIndex();

		    	if (modelIndex < 0)
		    		return;
		    	if (sortCol == modelIndex) {
		    		if ((sortCol == 6) && !isSortAsc)
		    			sortByID = !sortByID;
		    		isSortAsc = !isSortAsc;
		    	} else {
		    		sortCol = modelIndex;
		    		isSortAsc = true;
		    		sortByID = false;
		    	}

		    	for (int i = 0; i < getColumnCount(); i++) { 
		    		TableColumn column = colModel.getColumn(i);
		    		column.setHeaderValue(getColumnName(column.getModelIndex()));
		    	}
		    	table.getTableHeader().repaint();

		    	sortData(table);
		    }
		}

		class MyComparator implements Comparator<M44Setup> {
			protected boolean isSortAsc;
			protected int col;

			public MyComparator(boolean sortAsc, int col) {
				this.isSortAsc = sortAsc;
				this.col = col;
			}

			public int compare(M44Setup o1, M44Setup o2) {
				int result = 0;
				
				try {
					switch (col) {
					case 0:
						result = getScenTypeNumeric(o1.scentype) - getScenTypeNumeric(o2.scentype);
						break;
					case 1:
						result = o1.getConfigureName().compareTo(o2.getConfigureName()); break;
					case 2:
						String set1 = "";
						String set2 = "";
						if (o1.set != null)
							set1 = o1.set;
						if (o2.set != null)
							set2 = o2.set;
						result = set1.compareTo(set2); break;
					case 3:
						int f1 = getFrontNumeric(o1.front);
						int f2 = getFrontNumeric(o2.front);
						result = f1 - f2;
						break;
					case 4:
						result = o1.year.compareTo(o2.year);
						if (result == 0) {
							int m1 = getNumericMonth(o1.month);
							int m2 = getNumericMonth(o2.month);
							
							result = m1 - m2;
						}
						 break;
					case 5:
						result = o1.operation.compareTo(o2.operation); break;
					case 6:
						String code1 = o1.m44code;
						String code2 = o2.m44code;
						int c1 = code1.length();
						int c2 = code2.length();
						if (!sortByID && ((c1 != 0) && (c2 != 0)))
							result = code1.compareTo(code2);
						else if (!sortByID && ((c1 != 0) && (c2 == 0)))
							result = -1;
						else if (!sortByID && ((c1 == 0) && (c2 != 0)))
							result = 1;
						else {
							try {
								c1 = Integer.parseInt(o1.dowid);
							} catch (NumberFormatException ex) {
								c1 = 99999999;
							}
							try {
								c2 = Integer.parseInt(o2.dowid);
							} catch (NumberFormatException ex) {
								c2 = 99999999;
							}
							result = c1 - c2;
						}
						
						break;
					}
				} catch (NullPointerException ex) {

				}
				
				if (!isSortAsc)
					result = -result;
				
				return result;
			}

			private int getFrontNumeric(String front) {
				int value = 999;
				
				for (int i = 0; i < frontNames.length; i++)
					if (front.equals(frontNames[i]))
						value = i;
				
				return value;
			}

			public boolean equals(Object obj) {
				if (obj instanceof MyComparator) {
					MyComparator compObj = (MyComparator) obj;
					return compObj.isSortAsc == isSortAsc;
				}
				return false;
			}
			
			public int getScenTypeNumeric(String scentype) {
				int value = 999;
				
				for (int i = 0; i < typeNames.length; i++)
					if (scentype.equals(typeNames[i]))
						value = i;
				
				return value;
			}
			
			public int getNumericMonth(String month) {
				int value = 999;
				
				for (int i = 0; i < monthNames.length; i++)
					if (month.equals(monthNames[i]))
						value = i;
				
				return value;
			}
		}
	}
	
	public class URLCellRenderer extends JLabel implements TableCellRenderer {
	    // This method is called each time a cell in a column
	    // using this renderer needs to be rendered.
	    public Component getTableCellRendererComponent(JTable table, Object value,
	            boolean isSelected, boolean hasFocus, int rowIndex, int vColIndex) {
	        // 'value' is value contained in the cell located at
	        // (rowIndex, vColIndex)

	    	setForeground(new Color(0, 0, 255));
	    	
	        if (isSelected) {
	            // cell (and perhaps other cells) are selected
	        }

	        if (hasFocus) {
	            // this cell is the anchor and the table has the focus
	        }

	        // Configure the component with the specified value
	        setText(value.toString());

	        // Set tool tip if desired
	        setToolTipText((String)value);

	        // Since the renderer is a component, return itself
	        return this;
	    }

	    // The following methods override the defaults for performance reasons
	    public void validate() {}
	    public void revalidate() {}
	    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {}
	    public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {}
	}
}
